<?php

namespace Drupal\mailgroup_amazonses\Plugin\Mailgroup\Connection;

use Aws\S3\Exception\S3Exception;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\mailgroup\ConnectionBase;
use Drupal\mailgroup_amazonses\Traits\S3ClientFactoryTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Amazon SES connection plugin.
 *
 * @Connection(
 *   id = "amazonses",
 *   label = @Translation("Amazon SES")
 * )
 */
class AmazonSes extends ConnectionBase implements ContainerFactoryPluginInterface {

  use S3ClientFactoryTrait;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return (new static($configuration, $plugin_id, $plugin_definition))
      ->setS3ClientFactory($container->get('mailgroup_amazonses.s3_client_factory'));
  }

  /**
   * {@inheritdoc}
   */
  public function getFields() {
    $config = $this->configuration;

    $fields['region'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Region'),
      '#description' => $this->t('The AWS region SES is configured in.'),
      '#default_value' => $config['region'] ?? NULL,
      '#required' => TRUE,
    ];

    $fields['key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Key'),
      '#default_value' => $config['key'] ?? NULL,
    ];

    $fields['secret'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Secret'),
      '#default_value' => $config['secret'] ?? NULL,
    ];

    $fields['bucket'] = [
      '#type' => 'textfield',
      '#title' => $this->t('S3 Bucket'),
      '#description' => $this->t('The S3 bucket email are stored in.'),
      '#default_value' => $config['bucket'] ?? NULL,
    ];

    $fields['delete'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Delete after retrieval'),
      '#description' => $this->t('Delete messages from S3 after retrieval.'),
      '#default_value' => $config['delete'] ?? TRUE,
    ];

    return $fields;
  }

  /**
   * {@inheritdoc}
   */
  public function testConnection() {
    $config = [
      'region' => $this->configuration['region'],
      'credentials' => [
        'key' => $this->configuration['key'],
        'secret' => $this->configuration['secret'],
      ],
    ];

    $client = $this->clientFactory->create($config);

    try {
      $client->listObjects([
        'Bucket' => $this->configuration['bucket'],
        'MaxKeys' => 1,
      ]);

      return TRUE;
    }

    catch (S3Exception $e) {
      return FALSE;
    }
  }

}
